package pl.smi.sms.active.ra.ejbc;

import java.io.PrintWriter;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;

import javax.naming.NamingException;
import javax.resource.NotSupportedException;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterAssociation;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.ValidatingManagedConnectionFactory;
import javax.security.auth.Subject;

public class EJBCManagedConnectionFactoryImpl implements
		ManagedConnectionFactory, ValidatingManagedConnectionFactory,
		ResourceAdapterAssociation {
	private PrintWriter logWriter;
	private String activityCode, connectionData;
	private Integer version = 1;
	private EJBCResourceAdapterImpl resourceAdapter;

	
	
	public EJBCManagedConnectionFactoryImpl() {
		super();
		Logger.getLogger(this.getClass().getName()).info(
				"EJBCManagedConnectionFactoryImpl Constructor"+hashCode());
	}

	public Object createConnectionFactory() throws ResourceException {
		throw new NotSupportedException(
				"Non Managed Connection is not supported");
	}

	public Object createConnectionFactory(ConnectionManager connectionManager)
			throws ResourceException {
		if (connectionManager == null) {
			throw new ResourceAdapterInternalException(
					" connMngr == null Non Managed Connection is not supported");
		}
		return new EJBCConnectionFactoryImpl(this, connectionManager);
	}

	public ManagedConnection createManagedConnection(Subject subject,
			ConnectionRequestInfo connectionRequestInfo)
			throws ResourceException {

		Logger.getLogger(this.getClass().getSimpleName()).info(
				"createManagedConnection()");
		

		try {
			return new EJBCManagedConnectionImpl(this, connectionRequestInfo,
					subject);
		} catch (NamingException e) {
			throw new ResourceException(e);
		}

	}

	public PrintWriter getLogWriter() throws ResourceException {
		return logWriter;
	}

	public ManagedConnection matchManagedConnections(Set set, Subject subject,
			ConnectionRequestInfo connectionRequestInfo)
			throws ResourceException {
		Logger.getLogger(this.getClass().getSimpleName()).info(
				"matchManagedConnections()");

		for (Object connection : set) {
			if (connection instanceof EJBCManagedConnectionImpl
					&& this.equals(((EJBCManagedConnectionImpl) connection)
							.getManagedConnectionFactoryImpl())) {
				EJBCManagedConnectionImpl managedConnection = (EJBCManagedConnectionImpl) connection;
				return managedConnection;
			}
		}
		return null;
	}

	public void setLogWriter(PrintWriter logWriter) throws ResourceException {
		this.logWriter = logWriter;
	}

	public String getActivityCode() {
		return activityCode;
	}

	public void setActivityCode(String someString) {
		this.activityCode = someString;
	}

	public void setConnectionData(String connectionData) {
		this.connectionData = connectionData;
	}

	public String getConnectionData() {
		return connectionData;
	}

	public void setVersion(Integer version) {
		this.version = version;
	}

	public Integer getVersion() {
		return version;
	}

	public Set getInvalidConnections(Set set) throws ResourceException {
		Logger.getLogger(this.getClass().getName()).info(
				"getInvalidConnections");
		Set<EJBCManagedConnectionImpl> invalidConnections = new HashSet<EJBCManagedConnectionImpl>();
		for (Object object : set) {
			if (object instanceof EJBCManagedConnectionImpl) {
				EJBCManagedConnectionImpl mc = (EJBCManagedConnectionImpl) object;
				if (!mc.isValid()) {
					invalidConnections.add(mc);
				}
			}
			Logger.getLogger(this.getClass().getName()).info(
					object.getClass().getName());
		}
		if (invalidConnections.size() > 0) {
			return invalidConnections;
		} else {
			return null;
		}
	}

	public ResourceAdapter getResourceAdapter() {
		return resourceAdapter;
	}

	public void setResourceAdapter(ResourceAdapter resourceAdapter)
			throws ResourceException {
		Logger.getLogger(this.getClass().getName()).info("setResourceAdapter ra:"+resourceAdapter.hashCode()+ " this:"+this.hashCode());
		this.resourceAdapter = (EJBCResourceAdapterImpl)resourceAdapter;

	}
	
//	public EJBCActivityConnectionStateManager getConnectionStateManager() {
//		return this.resourceAdapter.getConnectionStateManager();
//	}
}
