/**
 * 
 */
package pl.smi.sms.active.ra.ejbc;

import java.io.PrintWriter;
import java.util.HashSet;
import java.util.logging.Logger;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.LocalTransaction;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.security.auth.Subject;
import javax.transaction.xa.XAResource;

import pl.smi.sms.active.process.ActivityProcessableRemote;

/**
 * @author rmalinowski
 * 
 */
public class EJBCManagedConnectionImpl implements ManagedConnection {
//		ActivityProcessListener {

	private PrintWriter logWriter;
	private final HashSet<ConnectionEventListener> eventListeners = new HashSet<ConnectionEventListener>();
	private final Subject subject;
	private final ConnectionRequestInfo connectionRequestInfo;
	private final EJBCManagedConnectionFactoryImpl managedConnectionFactoryImpl;
//	private final ActivitiesJDBCDAO actDAO;
//	private final ConnectionManager connectionManager;

	/**
	 * @param subject
	 * @param connectionRequestInfo
	 * @param managedConnectionFactoryImpl
	 * @throws NamingException
	 * 
	 */
	public EJBCManagedConnectionImpl(
			EJBCManagedConnectionFactoryImpl managedConnectionFactoryImpl,
			ConnectionRequestInfo connectionRequestInfo, Subject subject)
			throws NamingException {

		this.managedConnectionFactoryImpl = managedConnectionFactoryImpl;
		this.connectionRequestInfo = connectionRequestInfo;
		this.subject = subject;

//		connectionManager = new ConnectionManager(
//				((DataSource) new InitialContext()
//						.lookup(MappedNames.ACTIVE_DATASOURCE)));

//		this.actDAO = new ActivitiesJDBCDAO(connectionManager);
		Logger.getLogger(this.getClass().getName()).info(
				"EJBCManagedConnectionImpl Constructor "+hashCode());
	}

	public void addConnectionEventListener(ConnectionEventListener eventListener) {
		synchronized (eventListeners) {
			eventListeners.add(eventListener);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.resource.spi.ManagedConnection#associateConnection(java.lang.Object
	 * )
	 */
	public void associateConnection(Object arg0) throws ResourceException {
		Logger.getLogger(this.getClass().getSimpleName()).info(
				"associateConnection()");

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.resource.spi.ManagedConnection#cleanup()
	 */
	public void cleanup() throws ResourceException {
		Logger.getLogger(this.getClass().getSimpleName()).info("cleanup()");

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.resource.spi.ManagedConnection#destroy()
	 */
	public void destroy() throws ResourceException {
		Logger.getLogger(this.getClass().getSimpleName()).info("destroy()");

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.resource.spi.ManagedConnection#getConnection(javax.security.auth
	 * .Subject, javax.resource.spi.ConnectionRequestInfo)
	 */
	public Object getConnection(Subject arg0, ConnectionRequestInfo arg1)
			throws ResourceException {
		Logger.getLogger(this.getClass().getSimpleName()).info(
				"getConnection()");

//		ActivityProcessableRemote activityConnection = null;
//		try {
//			activityConnection = getManagedConnectionFactoryImpl()
//					.getConnectionStateManager().activateActivityConnection(
//							managedConnectionFactoryImpl.getActivityCode(),
//							managedConnectionFactoryImpl.getConnectionData());
//		} catch (SQLException e) {
//			throw new ResourceException(e);
//		}

		InitialContext ctx;
		try {
			// java.util.Properties p = new java.util.Properties();
			// p.put("org.omg.CORBA.ORBInitialHost", "localhost");
			// p.put("org.omg.CORBA.ORBInitialPort", "3700");
			
			ctx = new javax.naming.InitialContext();
			Object lookup = ctx
					.lookup("ejb/sms/active/module/EJBCActivityTest");
			ActivityProcessableRemote activity = (ActivityProcessableRemote) lookup;
			activity.doInit();
			EJBCConnectionImpl connectionImpl = new EJBCConnectionImpl(activity);
			
//			connectionImpl.addActivityProcessListener(this);
			return connectionImpl;
		} catch (NamingException e) {
			e.printStackTrace();
		}

		// EJBCConnectionImpl connectionImpl = new EJBCConnectionImpl(
		// activityInitiator.getActivity());
		
		return null;
	}

	// TODO: implement
	public boolean isValid() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.resource.spi.ManagedConnection#getLocalTransaction()
	 */
	public LocalTransaction getLocalTransaction() throws ResourceException {
		Logger.getLogger(this.getClass().getSimpleName()).info(
				"getLocalTransaction()");

		return null;
	}

	public PrintWriter getLogWriter() throws ResourceException {
		return logWriter;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.resource.spi.ManagedConnection#getMetaData()
	 */
	public ManagedConnectionMetaData getMetaData() throws ResourceException {
		Logger.getLogger(this.getClass().getSimpleName()).info("getMetaData()");

		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.resource.spi.ManagedConnection#getXAResource()
	 */
	public XAResource getXAResource() throws ResourceException {
		Logger.getLogger(this.getClass().getSimpleName()).info(
				"getXAResource()");

		return null;
	}

	public void removeConnectionEventListener(
			ConnectionEventListener eventListener) {
		synchronized (eventListeners) {
			eventListeners.remove(eventListener);
		}
	}

	public void setLogWriter(PrintWriter logWriter) throws ResourceException {
		this.logWriter = logWriter;
	}

	private void callListeners(int eventType) {
		synchronized (eventListeners) {
			ConnectionEvent event = new ConnectionEvent(this, eventType);
			for (ConnectionEventListener eventListener : eventListeners) {
				eventListener.connectionClosed(event);
			}
		}
	}

	public EJBCManagedConnectionFactoryImpl getManagedConnectionFactoryImpl() {
		return managedConnectionFactoryImpl;
	}

	public Subject getSubject() {
		return subject;
	}

	public ConnectionRequestInfo getConnectionRequestInfo() {
		return connectionRequestInfo;
	}

//	public void onProcessFailure(ActivityProcessable source,
//			ActivityRequest request) {
//		if (source instanceof EJBCConnectionImpl) {
//			EJBCConnectionImpl _source = (EJBCConnectionImpl) source;
//
//			// TODO
//		}
//
//	}

}
